﻿using System;
using System.Security.Permissions;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Security;
using Microsoft.SharePoint.Utilities;
using Microsoft.SharePoint.Workflow;
using Dienstfahrten.Logging;
using Dienstfahrten.Klassen;
using System.Security.Principal;

namespace Dienstfahrten.EventReceiverDienstfahrtList
{
    /// <summary>
    /// List Item Events
    /// </summary>
    public class EventReceiverDienstfahrtList : SPItemEventReceiver
    {
       /// <summary>
       /// An item is being added.
       /// </summary>
       public override void ItemAdding(SPItemEventProperties properties)
       {
           base.ItemAdding(properties);
           //EnsureItemSecurity(properties.ListItem);
       }

       /// <summary>
       /// An item is being updated.
       /// </summary>
       public override void ItemUpdating(SPItemEventProperties properties)
       {
           base.ItemUpdating(properties);
           //EnsureItemSecurity(properties.ListItem);
       }


       /// <summary>
       /// An item was added.
       /// </summary>
       public override void ItemAdded(SPItemEventProperties properties)
       {
           base.ItemAdded(properties);
           EnsureItemSecurity(properties.ListItem);
       }

       /// <summary>
       /// An item was updated.
       /// </summary>
       public override void ItemUpdated(SPItemEventProperties properties)
       {
           base.ItemUpdated(properties);
           EnsureItemSecurity(properties.ListItem);
       }

       /// <summary>
       /// Führt nötige Aktionen aus, um die korrekten Sicherheitseinstellungen eines
       /// News-Elements zu gewährleisten.
       /// </summary>
       /// <param name="item">Das <see cref="Microsoft.SharePoint.SPListItem"/>, welches
       /// ein News-Element darstellt und dessen Sicherheitseinstellungen überprüft und
       /// ggf. angepasst werden sollen.</param>
       /// http://www.microsoft.com/germany/msdn/solve/knowhow/howto/office/WieSetzeIchBerechtigungenAufListenUndListeneintraegeInSharePoint.mspx
       private static void EnsureItemSecurity(SPListItem item)
       {
           if(item==null) return;

           SPSecurity.RunWithElevatedPrivileges(
               delegate()
               {
                   var log = new Log();

                   using (var site = new SPSite(item.ParentList.ParentWeb.Site.ID))
                   {
                       using (SPWeb web = site.OpenWeb(item.ParentList.ParentWeb.ID))
                       {
                           try
                           {
                               //System.Threading.Thread.Sleep(5000);

                               //hole Item aus Web, damit man immer alle Berechtigungen von RunWithElevatedPrivileges darauf anwenden kann
                               SPListItem webItem = web.Lists[item.ParentList.ID].GetItemById(item.ID);

                               // Vererbung der Sicherheitseinstellungen deaktivieren.
                               if (!webItem.HasUniqueRoleAssignments) webItem.BreakRoleInheritance(true);

                               //Alle berechtigen User entfernen
                               foreach (SPRoleAssignment assignment in webItem.RoleAssignments)
                               {
                                   assignment.RoleDefinitionBindings.RemoveAll();
                                   assignment.Update();
                               }

                               // SiteCollection Admin mit Adminrechte hinzugefügen.
                               SPUserCollection authoriziedUsers = web.SiteAdministrators;
                               //authoriziedUsers.Add(athorLogin, string.Empty, string.Empty, string.Empty);
                               SPRoleDefinition AdminRole = web.RoleDefinitions.GetByType(SPRoleType.Administrator);
                               foreach (SPUser user in authoriziedUsers)
                               {
                                   var roleAssignment = new SPRoleAssignment(user.LoginName, string.Empty, string.Empty, string.Empty);
                                   roleAssignment.RoleDefinitionBindings.Add(AdminRole);
                                   webItem.RoleAssignments.Add(roleAssignment);
                               }

                               // User mit Contribute hinzugefügen.
                               //Hole Autor aus Item
                               SPUser athorLogin = new SPFieldUserValue(web, webItem["Author"].ToString()).User;
                               SPRoleDefinition ContributorRole = web.RoleDefinitions.GetByType(SPRoleType.Contributor);
                               var roleAssignment2 = new SPRoleAssignment(athorLogin.LoginName, string.Empty, string.Empty, string.Empty);
                               roleAssignment2.RoleDefinitionBindings.Add(ContributorRole);
                               webItem.RoleAssignments.Add(roleAssignment2);

                               log.Debug(
                                   "Stopped security inheritance for item #{0} ({1}) with permission level \"{2}\".",
                                   item.ID,
                                   item.ParentList.RootFolder.ServerRelativeUrl,
                                   ContributorRole.Name);
                           }
                           catch (Exception ex)
                            {
                                log.Error(
                                    "Can not modify security for item #{0} ({1}). An error occured.",
                                    item.ID,
                                    item.ParentList.RootFolder.ServerRelativeUrl);

                                log.Error(ex);
                            }
                        }
                    }
               });
       }


    }
}
