﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.SharePoint;
using Dienstfahrten.Logging;

namespace Dienstfahrten.Klassen
{
    class Dienstfahrt : IListenEintrag
    {
        public int ID {get; private set; }
        public DateTime? Datum { get; private set; }
        public string Von { get; private set; }
        public string Nach { get; private set; }
        public int Einfach { get; private set; }
        public int Gesamt { get; private set; }
        public bool Favorit { get; private set; }
        public bool Verschickt { get; private set; }
        public SPUser Benutzer { get; private set; }



        //Statischer Name der Liste
        public string ListName { get; private set; }
        public SPWeb currentWeb { get; private set; }
        public Log log { get; private set; }
        public SPList currentList { get; private set; }


        //Konstruktor
        public Dienstfahrt(SPWeb web) {
            this.ListName = "Dienstfahrten Liste";
            this.currentWeb = web;
            this.log = new Log();
            this.currentList = currentWeb.Lists[ListName];

            setDienstfahrtValues(new DateTime(), string.Empty, string.Empty, 0, 0, false, false);
        }



        public void setDienstfahrtValues(DateTime datum, string von, string nach, int einfach, int gesamt, bool fav, bool verschickt)
        {
            setValues(datum, von, nach, einfach, gesamt, fav, verschickt);
        }

        public void setValues(params Object[] values){
            
            try
            {
                if (validateData(values) == false) throw new Exception("Validation Error");

                this.Datum = (DateTime)values[0];
                this.Von = values[1].ToString();
                this.Nach = values[2].ToString();
                this.Einfach = (int)values[3];
                this.Gesamt = (int)values[4];
                this.Favorit = (bool)values[5];
                this.Verschickt = (bool)values[6];
            }
            catch (Exception ex)
            {
                log.Error(ex);
                throw;
            }

        }

        /// <summary>
        /// Speichere Objekt in SharePointListe
        /// </summary>
        /// <returns>Speicherung erfolgreich?</returns>
        public bool saveToList()
        {
            bool result = false;

            try
            {
                if (validateData(Datum, Von, Nach, Einfach, Gesamt, Favorit, Verschickt) == false) 
                    throw new Exception("Validation Error");

                SPListItem currentLI = null;

                //Wenn ein neues Item angelegt werden soll oder keines anhand der IssueID gefunden werden konnte
                currentLI = currentList.Items.Add();
                //Hole die neue ID
                currentLI.SystemUpdate();

                currentLI["Title"] = currentLI.ID;
                currentLI["Datum"] = this.Datum;
                currentLI["Von"] = this.Von;
                currentLI["Nach"] = this.Nach;
                currentLI["Einfach"] = this.Einfach;
                currentLI["Gesamt"] = this.Gesamt;
                currentLI["Favorit"] = this.Favorit;
                currentLI["Verschickt"] = this.Verschickt;

                //AllowUnsafeUpdates ist notwendig wenn ein Item nicht neu ist sondern die Werte aktualisiert werden.
                currentLI.Update();
                result = true;

            }
            catch (Exception ex)
            {
                result = false;
                //SPUtils.log(ex);
                log.Error(ex);
                throw;
            }

            return result;
        }

        /// <summary>
        /// Holt Eintrag aus der Liste
        /// </summary>
        /// <param name="id">ID des Eintrags</param>
        /// <returns>Eintrag in der Liste</returns>
        public Object getById(int id)
        {
            try
            {
                SPListItem currentLI = currentList.GetItemById(id); //Falls nicht vorhanden wird eine ArgumentException geworfen

                //Holt die Werte aus der SharePoint-Liste. Falls ein Wert leer ist wird NULL gesetzt oder ein leeres Objekt.

                this.ID = currentLI.ID;
                this.Datum = (currentLI["Datum"] != null) ? DateTime.Parse(currentLI["Datum"].ToString()) : new DateTime();
                this.Von = (currentLI["Von"] != null) ? currentLI["Von"].ToString() : null;
                this.Nach = (currentLI["Nach"] != null) ? currentLI["Nach"].ToString() : null;
                this.Einfach = (currentLI["Einfach"] != null) ? Convert.ToInt32(currentLI["Einfach"].ToString()) : new Int32();
                this.Gesamt = (currentLI["Gesamt"] != null) ? Convert.ToInt32(currentLI["Gesamt"].ToString()) : new Int32();
                this.Favorit = (currentLI["Favorit"] != null) ? bool.Parse(currentLI["Favorit"] + string.Empty) : new Boolean();
                this.Verschickt = (currentLI["Verschickt"] != null) ? bool.Parse(currentLI["Verschickt"] + string.Empty) : new Boolean();
                this.Benutzer = helper.getSPUserFromFieldId(SPBuiltInFieldId.Author, currentLI);
            }
            catch (ArgumentException aex)
            {
                //Wenn kein ListItem gefunden wurde
                log.Error(aex);
                return null;
            }
            catch (Exception ex)
            {
                log.Error(ex);
                throw;
            }
            
            return this;
        }

        /// <summary>
        /// Löscht eine Dienstfahrt anhand der ID
        /// </summary>
        /// <param name="id">ID der zu löschenden Dienstfahrt</param>
        /// <returns></returns>
        public bool deleteById(int id)
        {
            try
            {
                SPListItem item = currentList.GetItemById(id);
                item.Delete();
                return true;
            }
            catch (Exception ex)
            {
                log.Error(ex);
                throw;
            }
        }


        public bool validateData(params Object[] values)
        {
            bool result = false;

            try
            {
                int error = 0;
                if (values.Length > 7) throw new IndexOutOfRangeException("Allowed size is 7");
                string testString = "a";                

                if (values[0]== null || !Object.ReferenceEquals(values[0].GetType(), new DateTime().GetType())) error++;
                if (values[1] == null || !Object.ReferenceEquals(values[1].GetType(), testString.GetType())) error++;
                if (values[2] == null || !Object.ReferenceEquals(values[2].GetType(), testString.GetType())) error++;
                if (values[3] == null || !Object.ReferenceEquals(values[3].GetType(), new Int32().GetType())) error++;
                if (values[4] == null || !Object.ReferenceEquals(values[4].GetType(), new Int32().GetType())) error++;
                if (values[5] == null || !Object.ReferenceEquals(values[5].GetType(), new Boolean().GetType())) error++;
                if (values[6] == null || !Object.ReferenceEquals(values[6].GetType(), new Boolean().GetType())) error++;

                result = (error == 0);
            }
            catch (Exception ex)
            {
                log.Error(ex);
                result = false;
            }


            return result;
        }

        /// <summary>
        /// Speichere Dienstfahrt als Favorit
        /// </summary>
        /// <param name="id">id der Dienstfahrt, welche als Favorit verwendet werden soll</param>
        public void saveFavoritToList(int id)
        {
            this.getById(id);
            this.Datum = null;
            this.Favorit = true;
            this.saveToList();
        }

        /// <summary>
        /// Speichere Dienstfahrt als verschickt
        /// </summary>
        /// <param name="id">id der Dienstfahrt, welche als verschickt gekennzeichnet werden soll</param>
        public void saveVerschicktToList(int id) 
        {
            SPListItem item = currentList.GetItemById(id);
            item["Verschickt"] = true;
            item.Update();
        }

        
    }
}
